-- Lua FM widget
-- by Lothar Thole Apr 12 2024
-- 

local function create()
    return {value=nil, valuetext="", color0=lcd.RGB(0x30, 0x30, 0x30), color1=lcd.RGB(0x38, 0x38, 0x38), color2=lcd.RGB(0x40, 0x40, 0x40), color3=lcd.RGB(0x48, 0x48, 0x48), color4=lcd.RGB(0x50, 0x50, 0x50), color5=lcd.RGB(0x58, 0x58, 0x58), color6=lcd.RGB(0x60, 0x60, 0x60), color7=lcd.RGB(0x68, 0x68, 0x68), color8=lcd.RGB(0x70, 0x70, 0x70), source=nil, fontsize=FONT_XL, debugmode=false}
end

local function paint(widget)
    lcd.font(widget.fontsize)

    local w, h = lcd.getWindowSize()
    local text_w, text_h = lcd.getTextSize("")

    if widget.value == 0 then
        lcd.color(widget.color0)
    elseif widget.value == 1 then
        lcd.color(widget.color1)
    elseif widget.value == 2 then
        lcd.color(widget.color2)
    elseif widget.value == 3 then
        lcd.color(widget.color3)
    elseif widget.value == 4 then
        lcd.color(widget.color4)
    elseif widget.value == 5 then
        lcd.color(widget.color5)
    elseif widget.value == 6 then
        lcd.color(widget.color6)
    elseif widget.value == 7 then
        lcd.color(widget.color7)
    else
        lcd.color(widget.color8)
    end

    lcd.drawFilledRectangle(0, 0, w, h)
    lcd.color(BLACK)
    if widget.value ~= nil then
        --lcd.font(widget.fontsize)
        --lcd.drawText(w/2, (h - text_h)/2, widget.valuetext..widget.value, CENTERED)

        if widget.debugmode == true then
            --lcd.drawText(w / 2, (h - text_h)/ 2,"thr1 "..s1thr .." ;  ", RIGHT)
            --lcd.font(widget.fontsize)
            lcd.drawText(w/2, (h - text_h)/2, widget.value, CENTERED)
        else
            --lcd.font(widget.fontsize)
            lcd.drawText(w/2, (h - text_h)/2, widget.valuetext, CENTERED)
        end

    end
end

local function wakeup(widget)
    local source = system.getSource({category=CATEGORY_FLIGHT, member=FLIGHT_CURRENT_MODE})

    local newValue = nil
    if source ~= nil then
        newValue = source:value()
        newValueText = source:stringValue()
    end
    if widget.value ~= newValue then
        -- valuetemp = math.tointeger(newValue)
        -- widget.value = tostring(valuetemp)
        widget.value = math.tointeger(newValue)
        widget.valuetext = newValueText
        lcd.invalidate()
    end
end

local function configure(widget)
    -- FM0 Color
    line = form.addLine("FM0 Background Color")
    --form.addColorField(line, nil, function() return widget.r, widget.g, widget.b end, function(r, g, b) widget.r, widget.g, widget.b = r, g, b end)
    form.addColorField(line, nil, function() return widget.color0 end, function(color0) widget.color0 = color0 end)
    -- print("addColorField"..widget.color1) -- debug

    -- FM1 Color
    line = form.addLine("FM1 Background Color")
    --form.addColorField(line, nil, function() return widget.r, widget.g, widget.b end, function(r, g, b) widget.r, widget.g, widget.b = r, g, b end)
    form.addColorField(line, nil, function() return widget.color1 end, function(color1) widget.color1 = color1 end)
    -- print("addColorField"..widget.color1) -- debug

    -- FM2 Color
    line = form.addLine("FM2 Background Color")
    --form.addColorField(line, nil, function() return widget.r, widget.g, widget.b end, function(r, g, b) widget.r, widget.g, widget.b = r, g, b end)
    form.addColorField(line, nil, function() return widget.color2 end, function(color2) widget.color2 = color2 end)

    -- FM3 Color
    line = form.addLine("FM3 Background Color")
    --form.addColorField(line, nil, function() return widget.r, widget.g, widget.b end, function(r, g, b) widget.r, widget.g, widget.b = r, g, b end)
    form.addColorField(line, nil, function() return widget.color3 end, function(color3) widget.color3 = color3 end)

    -- FM4 Color
    line = form.addLine("FM4 Background Color")
    --form.addColorField(line, nil, function() return widget.r, widget.g, widget.b end, function(r, g, b) widget.r, widget.g, widget.b = r, g, b end)
    form.addColorField(line, nil, function() return widget.color4 end, function(color4) widget.color4 = color4 end)

    -- FM5 Color
    line = form.addLine("FM5 Background Color")
    --form.addColorField(line, nil, function() return widget.r, widget.g, widget.b end, function(r, g, b) widget.r, widget.g, widget.b = r, g, b end)
    form.addColorField(line, nil, function() return widget.color5 end, function(color5) widget.color5 = color5 end)

    -- FM6 Color
    line = form.addLine("FM6 Background Color")
    --form.addColorField(line, nil, function() return widget.r, widget.g, widget.b end, function(r, g, b) widget.r, widget.g, widget.b = r, g, b end)
    form.addColorField(line, nil, function() return widget.color6 end, function(color6) widget.color6 = color6 end)

    -- FM7 Color
    line = form.addLine("FM7 Background Color")
    --form.addColorField(line, nil, function() return widget.r, widget.g, widget.b end, function(r, g, b) widget.r, widget.g, widget.b = r, g, b end)
    form.addColorField(line, nil, function() return widget.color7 end, function(color7) widget.color7 = color7 end)

    -- FM8 Color
    line = form.addLine("FM8 Background Color")
    --form.addColorField(line, nil, function() return widget.r, widget.g, widget.b end, function(r, g, b) widget.r, widget.g, widget.b = r, g, b end)
    form.addColorField(line, nil, function() return widget.color8 end, function(color8) widget.color8 = color8 end)

    -- Font (XS, S, STD, L, L Bold, XL, XXL)
    line = form.addLine("Font Size")
    form.addChoiceField(line, nil, {{"Extra Small", FONT_XS}, {"Small", FONT_S}, {"Standard", FONT_STD}, {"Large", FONT_L}, {"Large Bold", FONT_L_BOLD}, {"Extra Large", FONT_XL}, {"Extra Extra Large", FONT_XXL}}, function() return widget.fontsize end, function(value) widget.fontsize = value end)

    -- Debug mode
    line = form.addLine("Display values for debug")
    form.addBooleanField(line, nil, function() return widget.debugmode end, function(value) widget.debugmode = value end)  
end


local function read(widget)
    widget.color0 = storage.read("color0")
    widget.color1 = storage.read("color1")
    widget.color2 = storage.read("color2")
    widget.color3 = storage.read("color3")
    widget.color4 = storage.read("color4")
    widget.color5 = storage.read("color5")
    widget.color6 = storage.read("color6")
    widget.color7 = storage.read("color7")
    widget.color8 = storage.read("color8")
    widget.debugmode = storage.read("debugmode")
end

local function write(widget)
    storage.write("color0", widget.color0)
    storage.write("color1", widget.color1)
    storage.write("color2", widget.color2)
    storage.write("color3", widget.color3)
    storage.write("color4", widget.color4)
    storage.write("color5", widget.color5)
    storage.write("color6", widget.color6)
    storage.write("color7", widget.color7)
    storage.write("color8", widget.color8)
    storage.write("debugmode", widget.debugmode)
end


local function init()
    system.registerWidget({key="FMode", name="Lua Flight Mode", create=create, paint=paint, wakeup=wakeup, configure=configure, read=read, write=write})
end

return {init=init}